# Generated by roxygen2: do not edit by hand

S3method(bake,step_collinear)
S3method(prep,step_collinear)
S3method(print,collinear_output)
S3method(print,collinear_selection)
S3method(summary,collinear_output)
S3method(summary,collinear_selection)
export(case_weights)
export(collinear)
export(collinear_select)
export(collinear_stats)
export(cor_clusters)
export(cor_cramer)
export(cor_df)
export(cor_matrix)
export(cor_select)
export(cor_stats)
export(drop_geometry_column)
export(f_auto)
export(f_auto_rules)
export(f_binomial_gam)
export(f_binomial_glm)
export(f_binomial_rf)
export(f_categorical_rf)
export(f_count_gam)
export(f_count_glm)
export(f_count_rf)
export(f_functions)
export(f_numeric_gam)
export(f_numeric_glm)
export(f_numeric_rf)
export(identify_categorical_variables)
export(identify_logical_variables)
export(identify_numeric_variables)
export(identify_response_type)
export(identify_valid_variables)
export(identify_zero_variance_variables)
export(model_formula)
export(preference_order)
export(score_auc)
export(score_cramer)
export(score_r2)
export(step_collinear)
export(target_encoding_lab)
export(target_encoding_loo)
export(target_encoding_mean)
export(target_encoding_rank)
export(validate_arg_df)
export(validate_arg_df_not_null)
export(validate_arg_encoding_method)
export(validate_arg_f)
export(validate_arg_function_name)
export(validate_arg_max_cor)
export(validate_arg_max_vif)
export(validate_arg_predictors)
export(validate_arg_preference_order)
export(validate_arg_quiet)
export(validate_arg_responses)
export(vif)
export(vif_df)
export(vif_select)
export(vif_stats)
importFrom(recipes,bake)
importFrom(recipes,prep)
