% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{declare_spatialData}
\alias{declare_spatialData}
\title{Declare spatialData}
\usage{
declare_spatialData()
}
\value{
dataset spatialData::vi_smol
}
\description{
Declare spatialData
}
\keyword{internal}
