% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.collinear_selection.R
\name{summary.collinear_selection}
\alias{summary.collinear_selection}
\title{Summarize single response selection results of \code{collinear}}
\usage{
\method{summary}{collinear_selection}(object = NULL, ...)
}
\arguments{
\item{object}{(sub-list in output of \code{\link[=collinear]{collinear()}} ) Object to summarize. Default: NULL}

\item{...}{Ignored, kept for consistency with generic.}
}
\value{
list: response name and character vector of selected predictors.
}
\description{
Summarize single response selection results of \code{collinear}
}
\seealso{
Other S3_methods:
\code{\link[=print.collinear_output]{print.collinear_output()}},
\code{\link[=print.collinear_selection]{print.collinear_selection()}},
\code{\link[=summary.collinear_output]{summary.collinear_output()}}
}
\concept{S3_methods}
