% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.collinear_selection.R
\name{print.collinear_selection}
\alias{print.collinear_selection}
\title{Print single selection results from \code{collinear}}
\usage{
\method{print}{collinear_selection}(x = NULL, n = 5, ...)
}
\arguments{
\item{x}{(required, sub-list in output of \code{\link[=collinear]{collinear()}}) Object to print. Default: NULL}

\item{n}{(optional, integer) Maximum printed vector length. Default: 5.}

\item{...}{Ignored, kept for consistency with generic.}
}
\description{
Print single selection results from \code{collinear}
}
\seealso{
Other S3_methods:
\code{\link[=print.collinear_output]{print.collinear_output()}},
\code{\link[=summary.collinear_output]{summary.collinear_output()}},
\code{\link[=summary.collinear_selection]{summary.collinear_selection()}}
}
\concept{S3_methods}
